/** 
 * <a href="http://servicemix.org">ServiceMix: The open source ESB</a> 
 * 
 * Copyright 2005 RAJD Consultancy Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/

package org.servicemix.jbi.messaging;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.net.URI;
import java.util.Collections;
import java.util.Map;
import java.util.Set;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.transaction.Transaction;
import javax.xml.namespace.QName;
import org.servicemix.jbi.framework.ComponentNameSpace;
import org.servicemix.jbi.servicedesc.ServiceEndpointImpl;
import EDU.oswego.cs.dl.util.concurrent.ConcurrentHashMap;

/**
 * ExchangePacket is responsible for carrying MessageExchange payloads
 * 
 * @version $Revision: 1.8 $
 */
public class ExchangePacket implements Externalizable {
    private static final long serialVersionUID = -9110837382914609624L;
    private static final String FAULT = "fault";
    private static final String IN = "in";
    private static final String OUT = "out";
    /**
     * provider role
     */
    public static final int PROVIDER_ROLE = 1;
    /**
     * consumer role
     */
    public static final int CONSUMER_ROLE = 2;
    private URI pattern;
    private String exchangeId;
    private ComponentNameSpace destinationId;
    private ComponentNameSpace sourceId;
    private String status;
    private QName serviceName;
    private QName interfaceName;
    private QName operationName;
    private Exception error;
    private Fault fault;
    private NormalizedMessage inMessage;
    private NormalizedMessage outMessage;
    private int role = PROVIDER_ROLE;
    private Map properties = new ConcurrentHashMap();
    private Map messages = new ConcurrentHashMap();
    private ServiceEndpointImpl endpoint;
    private boolean outbound;
    private transient Transaction transactionContext;
    private transient String endpointName;

    /**
     * @return Returns the outbound.
     */
    public boolean isOutbound() {
        return outbound;
    }

    /**
     * @param outbound The outbound to set.
     */
    public void setOutbound(boolean outbound) {
        this.outbound = outbound;
    }

    /**
     * @return Returns the endpoint.
     */
    public ServiceEndpointImpl getEndpoint() {
        return endpoint;
    }

    /**
     * @param endpoint The endpoint to set.
     */
    public void setEndpoint(ServiceEndpointImpl endpoint) {
        this.endpoint = endpoint;
    }

    /**
     * @return Returns the transactionContext.
     */
    public Transaction getTransactionContext() {
        return transactionContext;
    }

    /**
     * @param transactionContext The transactionContext to set.
     */
    public void setTransactionContext(Transaction transactionContext) {
        this.transactionContext = transactionContext;
    }

    /**
     * @return Returns the interfaceName.
     */
    public QName getInterfaceName() {
        return interfaceName;
    }

    /**
     * @param interfaceName The interfaceName to set.
     */
    public void setInterfaceName(QName interfaceName) {
        this.interfaceName = interfaceName;
    }

    /**
     * @return Returns the operationName.
     */
    public QName getOperationName() {
        return operationName;
    }

    /**
     * @param operationName The operationName to set.
     */
    public void setOperationName(QName operationName) {
        this.operationName = operationName;
    }

    /**
     * @return Returns the role.
     */
    public int getRole() {
        return role;
    }

    /**
     * @param role The role to set.
     */
    public void setRole(int role) {
        this.role = role;
    }

    /**
     * @return Returns the serviceName.
     */
    public QName getServiceName() {
        return serviceName;
    }

    /**
     * @param serviceName The serviceName to set.
     */
    public void setServiceName(QName serviceName) {
        this.serviceName = serviceName;
    }

    /**
     * @param status The status to set.
     */
    public void setStatus(String status) {
        this.status = status;
    }

    /**
     * @return the status
     */
    public String getStatus() {
        return status;
    }

    /**
     * @return Returns the pattern.
     */
    public URI getPattern() {
        return pattern;
    }

    /**
     * @param pattern The pattern to set.
     */
    public void setPattern(URI pattern) {
        this.pattern = pattern;
    }

    /**
     * @return Returns the error.
     */
    public Exception getError() {
        return error;
    }

    /**
     * @param error The error to set.
     */
    public void setError(Exception error) {
        this.error = error;
    }

    /**
     * @return Returns the exchangeId.
     */
    public String getExchangeId() {
        return exchangeId;
    }

    /**
     * @param exchangeId The exchangeId to set.
     */
    public void setExchangeId(String exchangeId) {
        this.exchangeId = exchangeId;
    }

    /**
     * @return Returns the fault.
     */
    public Fault getFault() {
        return fault;
    }

    /**
     * @param fault The fault to set.
     */
    public void setFault(Fault fault) {
        this.fault = fault;
        if (fault != null) {
            messages.put(FAULT, fault);
        }
    }

    /**
     * @return Returns the inMessage.
     */
    public NormalizedMessage getInMessage() {
        return inMessage;
    }

    /**
     * @param inMessage The inMessage to set.
     * @throws MessagingException
     */
    public void setInMessage(NormalizedMessage inMessage) throws MessagingException {
        if (this.inMessage != null && inMessage != null) {
            throw new MessagingException("InMessage already set");
        }
        this.inMessage = inMessage;
        if (inMessage != null) {
            messages.put(IN, inMessage);
        }
    }

    /**
     * @return Returns the messages.
     */
    public Map getMessages() {
        return messages;
    }

    /**
     * get a NormalizedMessage based on the message reference
     * 
     * @param name
     * @return a NormalizedMessage
     */
    public NormalizedMessage getMessage(String name) {
        return (NormalizedMessage) messages.get(name);
    }

    /**
     * set a NormalizedMessage with a named reference
     * 
     * @param message
     * @param name
     * @throws MessagingException
     */
    public void setMessage(NormalizedMessage message, String name) throws MessagingException {
        if (name.equals(IN)) {
            setInMessage(message);
        }
        else if (name.equals(OUT)) {
            setOutMessage(message);
        }
        else if (name.equals(FAULT)) {
            if (message instanceof Fault) {
                setFault((Fault) message);
            }
            else {
                throw new MessagingException("The fault message must implement the Fault interface, not: "
                        + message.getClass().getName());
            }
        }
        else {
            messages.put(name, message);
        }
    }

    /**
     * @return Returns the outMessage.
     */
    public NormalizedMessage getOutMessage() {
        return outMessage;
    }

    /**
     * @param outMessage The outMessage to set.
     * @throws MessagingException
     */
    public void setOutMessage(NormalizedMessage outMessage) throws MessagingException {
        if (this.outMessage != null && outMessage != null) {
            throw new MessagingException("OutMessage already set");
        }
        this.outMessage = outMessage;
        if (outMessage != null) {
            messages.put(OUT, outMessage);
        }
    }

    /**
     * @return Returns the properties.
     */
    public Map getProperties() {
        return properties;
    }

    /**
     * @param name
     * @return the proerty from the exchange
     */
    public Object getProperty(String name) {
        return properties.get(name);
    }

    /**
     * set a named property on the exchange
     * 
     * @param name
     * @param value
     */
    public void setProperty(String name, Object value) {
        properties.put(name, value);
    }

    /**
     * @return property names
     */
    public Set getPropertyNames() {
        return Collections.unmodifiableSet(properties.keySet());
    }

    /**
     * @return Returns the sourceId.
     */
    public ComponentNameSpace getSourceId() {
        return sourceId;
    }

    /**
     * @param sourceId The sourceId to set.
     */
    public void setSourceId(ComponentNameSpace sourceId) {
        this.sourceId = sourceId;
    }

    /**
     * @return Returns the destinationId.
     */
    public ComponentNameSpace getDestinationId() {
        return destinationId;
    }

    /**
     * @param destinationId The destinationId to set.
     */
    public void setDestinationId(ComponentNameSpace destinationId) {
        this.destinationId = destinationId;
    }

    /**
     * @return Returns the endpointName.
     */
    public String getEndpointName() {
        return endpointName;
    }

    /**
     * @param endpointName The endpointName to set.
     */
    public void setEndpointName(String endpointName) {
        this.endpointName = endpointName;
    }

    /**
     * @return pretty print
     */
    public String toString() {
        return "ExchangePacket[: id=" + exchangeId + ", serviceDest=" + serviceName + ",endpoint=" + endpoint + "]";
    }

    /**
     * Write to a Stream
     * @param out
     * @throws IOException
     */
    public void writeExternal(ObjectOutput out) throws IOException {
        out.writeBoolean(outbound);
        out.writeByte(role);
        out.writeObject(pattern);
        out.writeUTF(exchangeId != null ? exchangeId : "");
        out.writeUTF(status != null ? status : "");
        out.writeObject(destinationId);
        out.writeObject(sourceId);
        out.writeObject(serviceName);
        out.writeObject(interfaceName);
        out.writeObject(interfaceName);
        out.writeObject(error);
        out.writeObject(properties);
        out.writeObject(messages);
        out.writeObject(endpoint);
    }

    /**
     * Read from a stream
     * 
     * @param in
     * @throws IOException
     * @throws ClassNotFoundException
     */
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        outbound = in.readBoolean();
        role = in.readByte();
        pattern = (URI) in.readObject();
        exchangeId = in.readUTF();
        status = in.readUTF();
        destinationId = (ComponentNameSpace) in.readObject();
        sourceId = (ComponentNameSpace) in.readObject();
        serviceName = (QName) in.readObject();
        interfaceName = (QName) in.readObject();
        interfaceName = (QName) in.readObject();
        error = (Exception) in.readObject();
        properties = (Map) in.readObject();
        messages = (Map) in.readObject();
        endpoint = (ServiceEndpointImpl) in.readObject();
        fault = (Fault) messages.get(FAULT);
        inMessage = (NormalizedMessage) messages.get(IN);
        outMessage = (NormalizedMessage) messages.get(OUT);
    }
}